#!/usr/bin/env python
# -*- coding: utf_8 -*-

import string
import socket
import signal
import sys
import os
import time
import select

from GlobalVar import *
from UsersList import *
from AllUsersList import *
from RoomsList import *

from Room import *
from WaitRoom import *

# - Main -------------------------------------------------------

class ChatServer:
	def __init__(self):
		
		self.denylist = []
		
		f = open(DENYFILENAME, 'r')
		while 1:
			buf = f.readline(1024)
			
			if buf == '':
				break
				
			self.denylist.append(buf)	
		f.close()
		
		self.serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.serversocket.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR, 1) 
		self.serversocket.bind(("", PORT))
		self.serversocket.listen(5)
		
		write_log("listening on port" + str(PORT))

		self.clientsocket = {}
		
		self.sep		= SEP
		self.allusers	= AllUsersList(SEP, self)
		self.rooms		= Room(SEP, ROOMS_NOT_REMOVE, self)
		self.waitrooms	= WaitRoom(SEP, self)
		
	def removesocket(self, pConn):
		try:
			pConn.close()
		except:
			pass
			
		try:
			del self.clientsocket[pConn]
		except:
			pass
			
	def removeroom(self, roomid):
		if len(self.rooms.users(roomid)) == 0 and ROOMS_NOT_REMOVE < roomid:
			write_log(' ')
			self.rooms.remove(roomid)
			self.waitrooms.send_removeroom(roomid)
								
	def run(self):
		i = 0
		
		while 1:
			is_readable = [self.serversocket] + self.clientsocket.keys()
			is_writeable = []
			is_error = []
			
			r, w, e = select.select(is_readable, is_writeable, is_error, 1.0)
			
			if r:
				for fd in r:
					if fd == self.serversocket:
						
						conn, client_address = self.serversocket.accept()
						write_log("connection from " + client_address[0]);
						
						for ip in self.denylist:
							if client_address[0] == ip:
								write_log(" źε ༮")
								pConn.send("ALERT" + self.sep + "#" + self.sep + "#" + self.sep + "#" + self.sep + ' źεǾϴ' + '\n')
								conn.close()
						
						self.clientsocket[conn] = ['', int(time.time())]
					else:
						try:
							data = fd.recv(10240)
							
							if data:
								# Ͱ   ͸ óѴ
								self.handle(fd, '', data)
							else:
								raise socket.error
						except socket.error:
							self.removesocket(fd)
							write_log("")
						

							
			if e:
				for fd in e:
					self.removesocket(fd)
					write_log("")
				write_log("Error")
				
			i = i + 1
			
			if i == CHECK_TIME:
				i = 0
				self.connect_check()
			
	def user_disconnect(self, pUserid, pConn):
		write_log('ChatServer.py : user_disconnect')
		
		if self.allusers.has_key(pUserid):
		
			if self.allusers.get_roomid(pUserid):
						
				roomid = self.allusers.get_roomid(pUserid)
				
				if self.rooms.has_key(roomid):
					self.rooms.users(roomid).remove(pUserid)
	
					self.rooms.users(roomid).send_all(pUserid, self.allusers.get_name(pUserid), self.allusers.get_icon(pUserid), 'EXITUSER', '')
					
					self.removeroom(roomid)
								
			self.allusers.remove(pUserid)
		
		self.removesocket(pConn)
		
	def connect_check(self):
		for fd in self.clientsocket.keys():
			currenttime = int(time.time())
			
			if self.clientsocket[fd][1] + CHECK_TIME * 2 < currenttime:
				
				self.user_disconnect(self.clientsocket[fd][0], fd)
			else:
				try:
					fd.send('HEY' + self.sep + '#' + self.sep + '#' + self.sep + '#' + self.sep + '#' + '\n')
				except:
					self.removesocket(fd)
					
	def handle(self, pConn, pAddr, pMsg):
		data = string.split(string.strip(pMsg), SEP, 5)		
			
		try:
			command = data[0]
			userid	= data[1]
			name	= data[2]
			icon	= data[3]
			roomid	= int(data[4])
			msg		= data[5]
				
		except IndexError:
			write_log('ChatServer.py -> handle -> IndexError1 : ')
			self.removesocket(pConn)
			return
		except ValueError:
			write_log('ChatServer.py -> handle -> ValueError : ')
			self.removesocket(pConn)
			return
			
		tempstr = ''
		for temp in data:
			tempstr = tempstr + ' ' + temp
		
		## WaitRoom Handle ----------------------------------------------
		
		try:
			data = string.split(string.strip(msg), self.sep)
		except IndexError:
			write_log('ChatServer.py -> handle -> IndexError2 : Connection refuse')
			#self.removesocket(pConn)
			return
		
		msg = data[0]
		
		# ,   ɾ
		if command == "WHY":
			self.clientsocket[pConn][1] = int(time.time())
			return
			
		elif command == 'START':
			if self.waitrooms.start(pConn, pAddr, userid, name, icon) == 0:
				write_log(" ")
				try:
					self.clientsocket[pConn][0] = userid
				except:
					self.removesocket(pConn)
			return
			
		elif command == 'CONNECT':
			self.waitrooms.connect(pConn, pAddr, userid, name, icon)
			return
			
		elif command == 'DISCONNECT':
			if roomid == 0:
				self.waitrooms.disconnect(pConn, pAddr, userid, name, icon)
				self.removesocket(pConn)
			else:
				if self.rooms.has_key(roomid):
					self.rooms.users(roomid).remove(userid)
		
					self.rooms.users(roomid).send_all(userid, name, icon, 'EXITUSER', '')
					
					self.removeroom(roomid)
					
				# üƮ 
				self.allusers.remove(userid)
				
				self.removesocket(pConn)
			return
				
		elif command == 'MEMO':
			toid	= data[1]
			self.allusers.send_memo(userid, name, icon, msg, toid)
			return

		#  
		if roomid == 0:
			
			if command == 'ROOMLIST':
				self.waitrooms.send_roomlist(pConn, userid, name, icon)
				return
				
			elif command == 'MAKEROOM':
				roomname	= data[1]
				roomlimit	= int(data[2])
				roomtype	= data[3]
				roompass	= data[4]
				roomgroup	= data[5]
				self.waitrooms.makeroom(pConn, pAddr, roomid, userid, name, icon, roomname, roomlimit, roomtype, roompass, roomgroup)
				return
				
			elif command == 'JOINROOM':
				try:
					roomid		= int(data[1])
					self.waitrooms.joinroom(pConn, pAddr, roomid, userid, name, icon, roomid)
				except:
					pass
				return
				
			elif command == 'JOINSECRETROOM':
				try:
					roomid		= int(data[1])
					passwd		= data[2]
					self.waitrooms.joinsecretroom(pConn, pAddr, roomid, userid, name, icon, roomid, passwd)
				except:
					pass
				return
			else:
				write_log('ǿ ɾ ' + command)
				return
					
			
		# ȿ  
		else:
			print 'room id ' + str(roomid) + 'Դϴ'
			if roomid == '' and not self.rooms.has_key(roomid):
				write_log('߸ roomid')
				return
		
			# UserList Handle ------------
			
			if command == 'MSG':
				try:
				    fontcolor		= data[1]
				    backcolor		= data[2]
				    fontsize		= data[3]	 
				    fontname		= data[4]
				    fontbold		= data[5]
				    fontitalic		= data[6]
				    fontunderline	= data[7]
				
				    self.rooms.users(roomid).send_msg(userid, name, icon, msg, fontcolor, backcolor, fontsize, fontname, fontbold, fontitalic, fontunderline)
				except:
				    pass
				return
			
			elif command == 'TOMSG':
				to				= data[1]
				fontcolor		= data[2]
				backcolor		= data[3]
				fontsize		= data[4]	 
				fontname		= data[5]
				fontbold		= data[6]
				fontitalic		= data[7]
				fontunderline	= data[8]
				
				self.rooms.users(roomid).send_tomsg(userid, name, icon, msg, to, fontcolor, backcolor, fontsize, fontname, fontbold, fontitalic, fontunderline)
				return
			
			elif command == 'JOIN':
				self.rooms.users(roomid).send_all(userid, name, icon, 'JOINUSER', '')
				self.rooms.users(roomid).send_userlist(pConn, userid, name, icon)
				return
				
			elif command == 'EXITROOM':
				write_log('ϴ')
				self.rooms.users(roomid).remove(userid)
				self.rooms.users(roomid).send_all(userid, name, icon, 'EXITUSER', '')
				pConn.send('EXITROOMINIT' + self.sep + userid + self.sep + name + self.sep + icon + self.sep + '' + '\n')
			
				write_log('EXIT óԴϴ')
				self.allusers.set_roomid(userid, 0)
					
				self.removeroom(roomid)
				return
					
			elif command == 'CHANGENAME':
				write_log('ڸ ')
				newname		= data[1]
				newicon		= data[2]
				self.allusers.set_name(userid, newname)
				self.rooms.users(roomid).send_all(userid, name, icon, 'CHANGENAME', newname + self.sep + newicon)
				return
				
			elif command == 'INTRODUCE':
				write_log('Ұ')
				self.rooms.users(roomid).send_all(userid, name, icon, 'INTRODUCE', msg)
				return
				
			elif command == 'MUSIC':
				write_log('')
				self.rooms.users(roomid).send_all(userid, name, icon, 'MUSIC', msg)
				return
				
			elif command == 'GOOUT':
				toid	= data[1]
				self.rooms.users(roomid).send_to(userid, name, icon, 'GOOUT', '', toid)
				return
				
			elif command == 'FORCE':
				write_log('մϴ')
				self.rooms.users(roomid).remove(userid)
				self.rooms.users(roomid).send_all(userid, name, icon, 'FORCE', '')
				pConn.send('EXITROOMINIT' + self.sep + userid + self.sep + name + self.sep + icon + self.sep + '' + '\n')
				
				write_log('EXIT óԴϴ')
				self.allusers.set_roomid(userid, 0)
					
				self.removeroom(roomid)
				return
					
			elif command == 'CHANGEMASTER':
				write_log('  ')
				toid		= data[1]
				self.rooms.users(roomid).send_all(userid, name, icon, 'CHANGEMASTER', toid)
				return
	
			elif command == 'ROOMINFO' or command == 'MASTERROOMINFO':
				write_log(' ')
				roomid		= data[1]
				
				roomname	= self.rooms.get_roomname(int(roomid))
				roomlimit	= str(self.rooms.get_roomlimit(int(roomid)))
				roomtype	= self.rooms.get_roomtype(int(roomid))
				roompass	= self.rooms.get_roompass(int(roomid))
				roomgroup	= self.rooms.get_roomgroup(int(roomid))
				
				if roompass == '':
					roompass = 'null'
				
				tempstr = command + self.sep + userid + self.sep + name + self.sep + icon;
				tempstr = tempstr + self.sep + roomid;
				tempstr = tempstr + self.sep + roomname;
				tempstr = tempstr + self.sep + roomlimit;
				tempstr = tempstr + self.sep + roomtype;
				tempstr = tempstr + self.sep + roompass;
				tempstr = tempstr + self.sep + roomgroup;
				tempstr = tempstr + '\n';
				
				pConn.send(tempstr)
				write_log(tempstr)
				return
				
			elif command == 'CHANGEROOMINFO':
				write_log('  ')
				roomid		= int(data[1])
				roomname	= data[2]
				roomlimit	= data[3]
				roomtype	= data[4]
				roompass	= data[5]
				roomgroup	= data[6]
				
				oldroomname		= self.rooms.get_roomname(roomid)
				
				self.rooms.set_roomname(roomid, roomname)
				self.rooms.set_roomlimit(roomid, int(roomlimit))
				self.rooms.set_roomtype(roomid, roomtype)
				self.rooms.set_roompass(roomid, roompass)
				self.rooms.set_roomgroup(roomid, roomgroup)
				
				if roomname != oldroomname:
					write_log('ο ')
					self.rooms.users(roomid).send_all(userid, name, icon, 'CHANGEROOMNAME', roomname)
					
				return
					
			else:
				write_log('濡  ɾ ' + command)
				return
			
